#!/usr/bin/gawk -f
# Awk script to process .zil files
# Usage: 
#          awk -f getzil.awk <filename>.zil
#
#  or in unix:      
#
#          getzil.awk <filename>.zil
#
# will output to <filename>.txt
#
# Version 1.1
#
# This script might be supported.
# Send me any .zil files that trip it up
# Matt Davis - matt.davis@mrc-cbu.cam.ac.uk - 27.4.01
#
# This version modified not to output +Bit0


BEGIN {
	FS = " "
	OFS = "\t"
}

$1 == "Subject" {						# Read subject line
	gsub(/,/,"",$2)
	sub_num = $2
	start = 0
	scan_num == 0					# set scan number to zero

	for (field=1; field<=NF; field++) {     	# Find ID
		if (start == 1) {		
			id[sub_num] = id[sub_num] $field " "
		}
		if($field == "ID") {		
			start = 1				# rest of line is ID
		}
	}
}

$1 ~ /^[\*]+$/ { 						# set FILENAME; start to read data
	data = 0
	datafile = FILENAME 
}

$1 == "Item" {			 			# Read item heading to start collecting data
	gsub(/\,/,"",$2)					# get rid of comma from item number
	item = $2						# set item number
	if (item > max_item) {				# track maximum item number
		max_item = item
	}
	gsub(/\,/,"",$4)					# strip comma from clock on time
	cot[item,sub_num] = $4				# store clock on time

	if ($5 == "No") {					# if no response
		store_num_responses[item,sub_num]=0	# store as zero responses
	}		
	read = 0
}

$1 ~ /!/ {								# read item errors
	if(item_error==0) {                     		# add header for first error in each item
		error[sub_num] = "!  Error in subject number " sub_num " marked by !!"
	}
	error[sub_num] = error[sub_num] "\n" $0 		# keep error messages
	item_error = 1
}

$1 ~ /^[0-9]/ && read != 0 {					# if reading the second (or later) line of output

	prev_responses = num_responses			# store number of responses in previous line
	num_responses = prev_responses + NF			# add to previous number of responses

	if(num_responses > max_responses) {			# keep track of max number of responses
		max_responses = num_responses
	}	

	for (field=1; field<=NF; field++) {							# work through responses
		split($field,button_press,",")						# split RT from button presses
		store_num_responses[item,sub_num]=num_responses				# store new num_responses
		time[item,(field+prev_responses),sub_num]=button_press[1]		# store rt - with appropriate response number
		response[item,(field+prev_responses),sub_num]=button_press[2]	# store button

		if(button_press[2]=="+Bit0") {
			scan_num++
			scan_time_diff = (cot[item,sub_num] + button_press[1]) - scan_time
			scan_time = cot[item,sub_num] + button_press[1]
			store_tr[scan_time_diff,sub_num]++
		}
	}
	read++							# keep track of number of lines of responses read
}

$1 ~ /^[0-9]/ && read == 0 {					# if reading the first line of output

	num_responses = NF

	if(num_responses > max_responses) {			# keep track of max number of responses
		max_responses = num_responses
	}
	
	for (field=1; field<=NF; field++) {					# work through responses
		split($field,button_press,",")					# split RT from button presses
		store_num_responses[item,sub_num]=num_responses			# store num_responses
		time[item,field,sub_num]=button_press[1]				# store rt
		response[item,field,sub_num]=button_press[2]			# store button

		if(button_press[2]=="+Bit0") {
			scan_num++
			scan_time_diff = (cot[item,sub_num] + button_press[1]) - scan_time
			scan_time = cot[item,sub_num] + button_press[1]
			store_tr[scan_time_diff,sub_num]++
		}			
	}
	read = 1						# already read one line
}



END {
	num_fields = split(FILENAME,files,".")                       # set output filename
	for(i=1; i<num_fields; i++){
		output_file = output_file files[i] "."
	}
	output_file = output_file "txt"
	
	print "Output from DMDX file: "FILENAME > output_file

	for (subject=1;subject<=sub_num;subject++) {			# print subject number

		printf("\n") >> output_file
		printf("Sub_Num = ") >> output_file
		printf(subject OFS) >> output_file
		printf("Sub_ID = ") >> output_file
		printf(id[subject] OFS) >> output_file
		printf("\n") >> output_file


		for (item=0; item<=max_item; item++) {          	# for all possible items

			if((item,subject) in store_num_responses) {    	# if stored

				printf(item OFS) >> output_file 	# print item number

				temp = item SUBSEP subject
				if(temp in cot ) {					# if using RCOT
					printf(cot[item,subject] OFS) >> output_file	# print COT
				}

											# Ouput all +button press data
											# except +Bit0
				for (button=1; button<=store_num_responses[item,subject]; button++) {
					if(response[item,button,subject] ~ /^\+/ && response[item,button,subject] != "+Bit0") {

						gsub("+","",response[item,button,subject])	# delete + sign

						printf(response[item,button,subject] OFS) >> output_file
						printf(time[item,button,subject] OFS) >> output_file
					}
				}				
				printf("\n") >> output_file
			}
		}

		for (all in store_tr) {						# first pass to get rough average		
			total_tr = 0
			count_tr = 0
			split(all,tr_subject,SUBSEP)
			if(tr_subject[2]==subject) {
				total_tr += tr_subject[1] * store_tr[all]
				count_tr += store_tr[all]
			}
		}

		if(count_tr > 0) {
# commented out since it doesn't work very well!
#			print "Rough approximate mean TR for Subject " subject, total_tr / count_tr >> output_file
		}



		if (error[subject]!="") {
			print error[subject] >> output_file
		}
	}
}
				
				
				
